'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var reactMotion = require('react-motion');
var Resizable = _interopDefault(require('re-resizable'));
var ResizeObserver = _interopDefault(require('resize-observer-polyfill'));
var isEqual = _interopDefault(require('lodash.isequal'));
var debounce = _interopDefault(require('lodash.debounce'));

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var Pane = function (_React$Component) {
  inherits(Pane, _React$Component);

  function Pane() {
    classCallCheck(this, Pane);
    return possibleConstructorReturn(this, (Pane.__proto__ || Object.getPrototypeOf(Pane)).apply(this, arguments));
  }

  createClass(Pane, [{
    key: 'render',
    value: function render() {
      return React.createElement(
        'div',
        { className: this.props.className },
        this.props.children
      );
    }
  }]);
  return Pane;
}(React.Component);

Pane.defaultProps = {
  minWidth: 0,
  minHeight: 0,
  maxWidth: undefined,
  maxHeight: undefined,
  style: {},
  className: '',
  isResizable: {
    x: true,
    y: true,
    xy: true
  },
  children: ''
};

function reinsert(array, from, to) {
  var a = array.slice(0);
  var v = a[from];
  a.splice(from, 1);
  a.splice(to, 0, v);
  return a;
}

var directionDict = {
  right: 'x',
  bottom: 'y',
  bottomRight: 'xy'
};

var clamp = function clamp(n) {
  var min = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : n;
  var max = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : n;
  return Math.max(Math.min(n, max), min);
};

var springConfig = {
  damping: 30,
  stiffness: 500
};

var HYSTERESIS = 10;

var SortablePane = function (_React$Component) {
  inherits(SortablePane, _React$Component);

  function SortablePane(props) {
    classCallCheck(this, SortablePane);

    var _this = possibleConstructorReturn(this, (SortablePane.__proto__ || Object.getPrototypeOf(SortablePane)).call(this, props));

    _this.getSurroundingHalfSizes = function (currentPane, sizes, margin) {
      var nextPane = currentPane + 1;
      var prevPane = currentPane - 1;

      return sizes.reduce(function (sums, size, index) {
        var newSums = {};
        if (index < prevPane) {
          newSums.previous = sums.previous + size + margin;
        } else if (index === prevPane) {
          newSums.previous = sums.previous + size / 2;
        } else {
          newSums.previous = sums.previous;
        }

        if (index < nextPane) {
          newSums.next = sums.next + size + margin;
        } else if (index === nextPane) {
          newSums.next = sums.next + size / 2;
        } else {
          newSums.next = sums.next;
        }
        return newSums;
      }, { previous: 0, next: 0 });
    };

    _this.state = {
      delta: 0,
      mouse: 0,
      isPressed: false,
      lastPressed: 0,
      isResizing: false,
      panes: (_this.props.children || []).map(function (child) {
        return {
          id: child.props.id,
          width: child.props.width,
          height: child.props.height
        };
      })
    };
    _this.sizePropsUpdated = false;
    _this.handleTouchMove = _this.handleTouchMove.bind(_this);
    _this.handleMouseUp = _this.handleMouseUp.bind(_this);
    _this.handleMove = _this.handleMove.bind(_this);
    _this.debounceUpdate = debounce(_this.updateSize.bind(_this), 100);
    return _this;
  }

  createClass(SortablePane, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (typeof window !== 'undefined' && this.panes) {
        var _panes = this.panes;

        _panes.addEventListener('touchmove', this.handleTouchMove);
        _panes.addEventListener('touchend', this.handleMouseUp);
        _panes.addEventListener('mousemove', this.handleMove);
        _panes.addEventListener('mouseup', this.handleMouseUp);
        _panes.addEventListener('mouseleave', this.handleMouseUp);
        this.resizeObserver = new ResizeObserver(this.debounceUpdate);
      }
      if (this.panes && this.panes.parentElement instanceof Element) {
        this.resizeObserver.observe(this.panes.parentElement);
      }
      this.updateSize();
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      var panes = this.state.panes;

      var children = this.props.children || [];
      if (children.length > panes.length) return this.addPane();
      if (children.length < panes.length) return this.removePane();
      if (this.sizePropsUpdated) {
        this.sizePropsUpdated = false;
        this.updateSize();
      }
      return undefined;
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (typeof window !== 'undefined' && this.panes) {
        var _panes2 = this.panes;

        _panes2.removeEventListener('touchmove', this.handleTouchMove);
        _panes2.removeEventListener('touchend', this.handleMouseUp);
        _panes2.removeEventListener('mousemove', this.handleMove);
        _panes2.removeEventListener('mouseup', this.handleMouseUp);
        if (this.panes && this.panes.parentElement instanceof Element) {
          this.resizeObserver.unobserve(this.panes.parentElement);
        }
      }
    }
  }, {
    key: 'onResize',
    value: function onResize(i, e, dir, elementRef, delta) {
      var _this2 = this;

      var panes = this.state.panes;

      panes = panes.map(function (pane, index) {
        if (_this2.order.indexOf(i) === index && _this2.panes) {
          var _panes$children$i = _this2.panes.children[i],
              offsetWidth = _panes$children$i.offsetWidth,
              offsetHeight = _panes$children$i.offsetHeight;

          return {
            width: offsetWidth,
            height: offsetHeight,
            id: pane.id
          };
        }
        return pane;
      });
      this.setState({ panes: panes });
      var pane = panes[this.order.indexOf(i)];
      if (!pane) return;
      if (!this.props.onResize) return;
      var direction = void 0;
      if (dir === 'right' || dir === 'bottom' || dir === 'bottomRight') {
        direction = directionDict[dir];
      } else {
        return;
      }
      this.props.onResize(e, pane.id, panes, {
        pane: pane,
        direction: direction,
        delta: delta
      });
    }
  }, {
    key: 'getPanePropsArrayOf',
    value: function getPanePropsArrayOf(key) {
      return this.state.panes.map(function (pane) {
        return pane['' + key];
      });
    }
  }, {
    key: 'getPaneSizeList',
    value: function getPaneSizeList() {
      var width = this.getPanePropsArrayOf('width');
      var height = this.getPanePropsArrayOf('height');
      return this.isHorizontal() ? width : height;
    }

    /**
     * Find the position sum of halfway points of panes surrounding a given pane
     *
     *  |-------------|
     *  |             | ---> 'previous' halfway
     *  |-------------|
     *                  <--- margin
     *  |-------------|
     *  | currentPane |
     *  |-------------|
     *                  <--- margin
     *  |-------------|
     *  |             |
     *  |             | ---> 'next' halfway
     *  |             |
     *  |-------------|
     *
     *
     * @param  {number}   currentPane - Index of rerference pane
     * @param  {number[]} sizes       - Array of pane sizes
     * @param  {number}   margin      - The margin between panes
     * @return {object}               - Object containing 'prevoius' and 'next'
     *                                  pane halfway points
     */

  }, {
    key: 'getItemCountByPosition',


    /**
     * Determine where a particular pane should be ordered
     *
     * @param  {number} position     - Top of the current pane
     * @param  {number} paneIndex    - Index of the pane
     * @return {number}              - New index of the pane based on position
     */
    value: function getItemCountByPosition(position, paneIndex) {
      var size = this.getPaneSizeList();
      var margin = this.props.margin;

      var halfSizes = this.getSurroundingHalfSizes(paneIndex, size, margin || 0);
      if (position + size[paneIndex] > halfSizes.next + HYSTERESIS) return paneIndex + 1;
      if (position < halfSizes.previous - HYSTERESIS) return paneIndex - 1;
      return paneIndex;
    }

    /**
     * Update pane size
     */

  }, {
    key: 'updateSize',
    value: function updateSize() {
      if (!this.panes || !this.panes.children) return;
      var panes = this.panes;

      var newPanes = (this.props.children || []).map(function (child, i) {
        var _panes$children$i2 = panes.children[i],
            offsetWidth = _panes$children$i2.offsetWidth,
            offsetHeight = _panes$children$i2.offsetHeight;

        return {
          id: child.props.id,
          width: offsetWidth,
          height: offsetHeight
        };
      });
      if (!isEqual(newPanes, this.state.panes)) this.setState({ panes: newPanes });
    }
  }, {
    key: 'getItemPositionByIndex',
    value: function getItemPositionByIndex(index) {
      var size = this.getPaneSizeList();
      var sum = 0;
      if (size.some(function (s) {
        return typeof s === 'string';
      })) return 0;
      for (var i = 0; i < index; i += 1) {
        sum += size[i] + this.props.margin;
      }
      return sum;
    }
  }, {
    key: 'isHorizontal',
    value: function isHorizontal() {
      return this.props.direction === 'horizontal';
    }
  }, {
    key: 'addPane',
    value: function addPane() {
      var _this3 = this;

      if (!this.panes || !this.panes.children) return;
      var newPanes = this.state.panes;
      var children = this.props.children || [];
      children.forEach(function (child, i) {
        var ids = _this3.state.panes.map(function (pane) {
          return pane.id;
        });
        if (ids.indexOf(child.props.id) === -1) {
          var _id = child.props.id;
          var _panes3 = _this3.panes;

          if (!_panes3) return;

          var _panes3$children$i$ge = _panes3.children[i].getBoundingClientRect(),
              _width = _panes3$children$i$ge.width,
              _height = _panes3$children$i$ge.height;

          var _pane = { id: _id, width: _width, height: _height };
          newPanes.splice(i, 0, _pane);
        }
      });
      this.setState({ panes: newPanes });
    }
  }, {
    key: 'removePane',
    value: function removePane() {
      var _this4 = this;

      var newPanes = this.state.panes;
      this.state.panes.forEach(function (pane, i) {
        var ids = (_this4.props.children || []).map(function (child) {
          return child.props.id;
        });
        if (ids.indexOf(pane.id) === -1) {
          newPanes.splice(i, 1);
        }
      });
      this.setState({ panes: newPanes });
    }
  }, {
    key: 'handleResizeStart',
    value: function handleResizeStart(i, e) {
      var panes = this.state.panes;

      this.setState({ isResizing: true });
      var id = panes[this.order.indexOf(i)].id;

      if (typeof id === 'undefined') return;
      if (this.props.onResizeStart) {
        this.props.onResizeStart(e, id, panes);
      }
    }
  }, {
    key: 'handleResizeStop',
    value: function handleResizeStop(i, e, dir, elementRef, delta) {
      var panes = this.state.panes;

      this.setState({ isResizing: false });
      var pane = panes[this.order.indexOf(i)];
      var id = pane.id;

      if (typeof id === 'undefined') return;
      if (this.props.onResizeStop) {
        var _direction = void 0;
        if (dir === 'right' || dir === 'bottom' || dir === 'bottomRight') {
          _direction = directionDict[dir];
        } else {
          return;
        }
        this.props.onResizeStop(e, id, panes, {
          pane: pane,
          direction: _direction,
          delta: delta
        });
      }
    }
  }, {
    key: 'handleMouseDown',
    value: function handleMouseDown(pos, pressX, pressY, e) {
      if (this.props.dragHandleClassName) {
        if (e.target instanceof HTMLElement) {
          if (!e.target.classList.contains(this.props.dragHandleClassName)) {
            return;
          }
        }
      }
      var delta = void 0;
      if (window && window.TouchEvent && e.nativeEvent instanceof window.TouchEvent) {
        var event = e.nativeEvent.touches[0];
        delta = this.isHorizontal() ? event.pageX - pressX : event.pageY - pressY;
      } else if (window && window.MouseEvent && e.nativeEvent instanceof window.MouseEvent) {
        var _event = e.nativeEvent;
        delta = this.isHorizontal() ? _event.pageX - pressX : _event.pageY - pressY;
      }
      this.setState({
        delta: delta,
        mouse: this.isHorizontal() ? pressX : pressY,
        isPressed: true,
        lastPressed: pos
      });
      if (!this.props.children) return;
      var child = this.props.children[pos];
      if (!child) return;
      if (this.props.onDragStart) {
        this.props.onDragStart(e, child.props.id, this.state.panes);
      }
    }
  }, {
    key: 'handleMove',
    value: function handleMove(_ref) {
      var pageX = _ref.pageX,
          pageY = _ref.pageY;
      var _state = this.state,
          isPressed = _state.isPressed,
          delta = _state.delta,
          lastPressed = _state.lastPressed,
          isResizing = _state.isResizing,
          panes = _state.panes;

      if (isPressed && !isResizing) {
        var _mouse = this.isHorizontal() ? pageX - delta : pageY - delta;

        var _ref2 = this.props.children || [],
            length = _ref2.length;

        var newPosition = this.getItemCountByPosition(_mouse, this.order.indexOf(lastPressed));
        var pos = clamp(Math.round(newPosition), 0, length - 1);
        var _newPanes = reinsert(panes, this.order.indexOf(lastPressed), pos);
        this.setState({ mouse: _mouse, panes: _newPanes });
        if (!this.props.onOrderChange) return;
        if (!isEqual(panes, _newPanes)) {
          if (this.props.onOrderChange) {
            this.props.onOrderChange(panes, _newPanes);
          }
        }
      }
    }
  }, {
    key: 'handleTouchStart',
    value: function handleTouchStart(key, x, y, e) {
      this.handleMouseDown(key, x, y, e);
    }
  }, {
    key: 'handleTouchMove',
    value: function handleTouchMove(e) {
      e.preventDefault();
      this.handleMove(e.touches[0]);
    }
  }, {
    key: 'handleMouseUp',
    value: function handleMouseUp(e) {
      var children = this.props.children || [];
      if (children.length === 0) return;
      this.setState({ isPressed: false, delta: 0 });
      var child = children[this.state.lastPressed];
      var lastPressedId = child.props.id;
      if (!this.props.isSortable) return;
      if (this.props.onDragStop) {
        this.props.onDragStop(e, lastPressedId, this.state.panes);
      }
    }
  }, {
    key: 'renderPanes',
    value: function renderPanes() {
      var _this5 = this;

      var _state2 = this.state,
          mouse = _state2.mouse,
          isPressed = _state2.isPressed,
          lastPressed = _state2.lastPressed,
          isResizing = _state2.isResizing;
      var _props = this.props,
          disableEffect = _props.disableEffect,
          isSortable = _props.isSortable;

      var children = this.props.children || [];
      return children.map(function (child, i) {
        var pos = _this5.getItemPositionByIndex(_this5.order.indexOf(i));
        var springPosition = reactMotion.spring(pos, springConfig);
        var style = lastPressed === i && isPressed && !isResizing ? {
          scale: disableEffect ? 1 : reactMotion.spring(1.05, springConfig),
          shadow: disableEffect ? 0 : reactMotion.spring(16, springConfig),
          x: _this5.isHorizontal() ? mouse : 0,
          y: !_this5.isHorizontal() ? mouse : 0
        } : {
          scale: disableEffect ? 1 : reactMotion.spring(1, springConfig),
          shadow: disableEffect ? 0 : reactMotion.spring(0, springConfig),
          x: _this5.isHorizontal() ? springPosition : 0,
          y: !_this5.isHorizontal() ? springPosition : 0
        };
        return React.createElement(
          reactMotion.Motion,
          { style: style, key: child.props.id },
          function (_ref3) {
            var scale = _ref3.scale,
                shadow = _ref3.shadow,
                x = _ref3.x,
                y = _ref3.y;

            var onResize = _this5.onResize.bind(_this5, i);
            var onMouseDown = isSortable ? _this5.handleMouseDown.bind(_this5, i, x, y) : function () {
              return null;
            };
            var onTouchStart = _this5.handleTouchStart.bind(_this5, i, x, y);
            var onResizeStart = _this5.handleResizeStart.bind(_this5, i);
            var onResizeStop = _this5.handleResizeStop.bind(_this5, i);
            var userSelect = isPressed || isResizing ? {
              userSelect: 'none',
              MozUserSelect: 'none',
              WebkitUserSelect: 'none',
              MsUserSelect: 'none'
            } : {
              userSelect: 'auto',
              MozUserSelect: 'auto',
              WebkitUserSelect: 'auto',
              MsUserSelect: 'auto'
            };

            // take a copy rather than direct-manipulating the child's prop, which violates React
            // and causes problems if the child's prop is a static default {}, which then will be
            // shared across all children!
            var customStyle = Object.assign({}, child.props.style);
            Object.assign(customStyle, {
              boxShadow: 'rgba(0, 0, 0, 0.2) 0px ' + shadow + 'px ' + 2 * shadow + 'px 0px',
              transform: 'translate3d(' + x + 'px, ' + y + 'px, 0px) scale(' + scale + ')',
              WebkitTransform: 'translate3d(' + x + 'px, ' + y + 'px, 0px) scale(' + scale + ')',
              MozTransform: 'translate3d(' + x + 'px, ' + y + 'px, 0px) scale(' + scale + ')',
              MsTransform: 'translate3d(' + x + 'px, ' + y + 'px, 0px) scale(' + scale + ')',
              zIndex: i === lastPressed && _this5.state.isPressed ? 999999 : child.props.style && child.props.style.zIndex || 'auto',
              position: 'absolute'
            }, userSelect);

            var extendsProps = {
              onMouseDown: onMouseDown,
              onTouchStart: onTouchStart
            };

            return React.createElement(
              Resizable,
              _extends({
                className: child.props.className,
                onResize: onResize,
                enable: {
                  top: false,
                  right: child.props.isResizable.x,
                  bottomRight: child.props.isResizable.xy,
                  bottom: child.props.isResizable.y,
                  left: false,
                  topRight: false,
                  bottomLeft: false,
                  topLeft: false
                },
                defaultSize: {
                  width: child.props.width,
                  height: child.props.height
                },
                minWidth: child.props.minWidth,
                minHeight: child.props.minHeight,
                maxWidth: child.props.maxWidth,
                maxHeight: child.props.maxHeight,
                style: customStyle,
                onResizeStart: onResizeStart,
                onResizeStop: onResizeStop,
                grid: _this5.props.grid
              }, extendsProps),
              child.props.children
            );
          }
        );
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this6 = this;

      var _props2 = this.props,
          style = _props2.style,
          className = _props2.className;

      return React.createElement(
        'div',
        {
          ref: function ref(c) {
            _this6.panes = c;
          },
          className: className,
          style: _extends({ position: 'relative', height: '100%' }, style)
        },
        this.renderPanes()
      );
    }
  }, {
    key: 'order',
    get: function get$$1() {
      var children = this.props.children || [];
      return this.state.panes.map(function (p) {
        return children.findIndex(function (c) {
          return p.id === c.props.id;
        });
      });
    }
  }]);
  return SortablePane;
}(React.Component);

SortablePane.defaultProps = {
  direction: 'horizontal',
  style: {},
  children: [],
  margin: 0,
  onClick: function onClick() {
    return null;
  },
  onTouchStart: function onTouchStart() {
    return null;
  },
  onResizeStart: function onResizeStart() {
    return null;
  },
  onResize: function onResize() {
    return null;
  },
  onResizeStop: function onResizeStop() {
    return null;
  },
  onDragStart: function onDragStart() {
    return null;
  },
  onDragStop: function onDragStop() {
    return null;
  },
  onOrderChange: function onOrderChange() {
    return null;
  },
  className: '',
  disableEffect: false,
  isSortable: true
};

exports.Pane = Pane;
exports.SortablePane = SortablePane;
//# sourceMappingURL=react-sortable-pane.es5.js.map
